/*
imatrix.c

matrix manipulation library
version 0.05
Peter A. Cejchan, GPL
Mon Jul 31 12:24:25 MET 2006
<cej@gli.cas.cz | pac7@post.cz>

int and bool matrix structure and functions

*/


#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>
#include "libmatrix.h" 
#include "imatrix.h" 


// functions:

iMatrix *
imat_alloc(int rows, int cols){
// allocates memory for a matrix

	int i,j;
	iMatrix *M;
	M=(iMatrix*)malloc(sizeof(iMatrix));
	M->rows=rows; M->cols=cols;
	M->x=(int**)malloc(M->rows*sizeof(int*));
	for(j=0;j<rows;j++) M->x[j]=(int*)malloc(cols*sizeof(int));
	for(i=0;i<M->rows;i++)
		for(j=0;j<M->cols;j++) M->x[i][j]=0;
	return M;
}


int
imat_read(iMatrix *M)
{
// reads in a whitespace and newline separated matrix

	char *buf, *av[MAXCOLS];
	int i, j, n, first;


	for(i=0; i<M->rows; i++){
	n=0;
	while(n==0){
		if((buf = Brdstr(&bin, '\n', 1)) == nil) return 0;			// nothing to read
		first=buf[0];
//				print("first==%c\n", first);
		n=tokenize(buf, av, M->cols);
		if(n!=0){
			// skip comment lines
			if((first=='#') || (first=='>')){
//				print("comments\n");
				n=0;
			}
		}
		else{
		// skip whitespace lines
//			print("whitespace\n");	
		}
	}	// ends up with n > 0
			for(j=0; j<M->cols; j++)
				M->x[i][j]=atoi(av[j]);
			free(buf);
	}
	return 1;
}

int
tagged_imat_read(iMatrix *M){
// reads in sparse matrix in tagged format, i.e., coordinates of non-empty elements, followed by int value; newline separated elements

	char *buf, *av[MAXCOLS];
	int i, j, n;
// print("%d %d", M->rows , M->cols);
	for(i=0; i<M->rows; i++)
		for(j=0; j<M->cols; j++)
			M->x[i][j]=0;

		while((buf = Brdstr(&bin, '\n', 1)) != nil){	
			// skip comment lines
			while((buf[0]=='#') || (buf[0]=='>')){

			//	print("comments\n");	
				if((buf = Brdstr(&bin, '\n', 1)) == nil) return 1;	// read another line
			}	
			n=tokenize(buf, av, M->cols);  //parse line into an array 'av'
			// skip whitespace lines
			while(n==0){
			//	print("whitespace\n");	
				if((buf = Brdstr(&bin, '\n', 1)) == nil) return 1;	// read another line
				n=tokenize(buf, av, M->cols);
			}	

			i=atoi(av[0]);
			j=atoi(av[1]);
			M->x[i][j]=atoi(av[2]);
			free(buf);
		}
		return 1;
}

int
tagged_boolmat_read(iMatrix *M){
// reads in sparse matrix in tagged format, i.e., coordinates of non-empty elements, newline separated

	char *buf, *av[MAXCOLS];
	int i, j, n;
	for(i=0; i<M->rows; i++)
		for(j=0; j<M->cols; j++)
			M->x[i][j]=0;

		while((buf = Brdstr(&bin, '\n', 1)) != nil){	
			// skip comment lines
			while((buf[0]=='#') || (buf[0]=='>')){

			//	print("comments\n");	
				if((buf = Brdstr(&bin, '\n', 1)) == nil) return 1;	// read another line
			}	
			n=tokenize(buf, av, 2);  //parse line into an array 'av'
			// skip whitespace lines
			while(n==0){
			//	print("whitespace\n");	
				if((buf = Brdstr(&bin, '\n', 1)) == nil) return 1;	// read another line
				n=tokenize(buf, av, 2);
			}	

			i=atoi(av[0]);
			j=atoi(av[1]);
			M->x[i][j]=1;	//  1 stands for TRUE at position i, j
//print("%d %d\n", i, j);
			free(buf);
		}
		return 1;
}

int
line_tagged_boolmat_read(iMatrix *M){
// reads in sparse matrix in line tagged format, i.e., lines of column-coordinates of non-empty elements, newline separated; empty lines mean rows of zeros !! Comments maty precede header, and/or data, and must start with  '#', or '>'. No comments are allowed within the data.

	char *buf, *av[MAXCOLS];
	int i, j, k, n;
	for(i=0; i<M->rows; i++)
		for(j=0; j<M->cols; j++)
			M->x[i][j]=0;
	if((buf = Brdstr(&bin, '\n', 1)) == nil) return 0;		// nothing to read
		// skip comment lines
		while((buf[0]=='#') || (buf[0]=='>')){

//			print("comments\n");	
			if((buf = Brdstr(&bin, '\n', 1)) == nil) return 0;	// read another line, if you can
		}

	for(i=0; i<M->rows; i++){
		if(i!=0)	{
			if((buf = Brdstr(&bin, '\n', 1)) == nil) return 0;	// read another line, if you can
}
			n=tokenize(buf, av, MAXCOLS);  //parse line into an array 'av'
		for(j=0; j<n; j++){
			k=atoi(av[j]);
//			print("k=%d \n", k);
			M->x[i][k]=1;	//  1 stands for TRUE at position i, j
//			print("%d %d\n", i, k);
		}
		free(buf);
	}
	return 1;
}


int
line_tagged_imat_read(iMatrix *M){
// reads in sparse matrix in line tagged format, i.e., lines of column-coordinates of non-empty elements, newline separated; empty lines mean rows of zeros !! Comments maty precede header, and/or data, and must start with  '#', or '>'. No comments are allowed within the data.

	char *buf, *av[MAXCOLS];
	int i, j, k, n;
	for(i=0; i<M->rows; i++)
		for(j=0; j<M->cols; j++)
			M->x[i][j]=0;
	if((buf = Brdstr(&bin, '\n', 1)) == nil) return 0;		// nothing to read
		// skip comment lines
		while((buf[0]=='#') || (buf[0]=='>')){

//			print("comments\n");	
			if((buf = Brdstr(&bin, '\n', 1)) == nil) return 0;	// read another line, if you can
		}

	for(i=0; i<M->rows; i++){
		if(i!=0)	{
			if((buf = Brdstr(&bin, '\n', 1)) == nil) return 0;	// read another line, if you can
}
			n=tokenize(buf, av, MAXCOLS);  //parse line into an array 'av'
		for(j=0; j<n; j++){
			k=atoi(av[j]);
//			print("k=%d \n", k);
			M->x[i][k]=atoi(av[2]);	//  reads in a value at position i, j
//			print("%d %d\n", i, k);
		}
		free(buf);
	}
	return 1;
}

void 
imat_write(iMatrix *M){
//writes out a matrix 
	int i, j;
	print("# rows, cols:\n%d %d\n", M->rows, M->cols);
	print("# data:\n");
	for(i=0; i<M->rows; i++){
		for(j=0; j<M->cols; j++){
			print("%d ", M->x[i][j]);
		}
		print("\n");
	}
	return;
}


int 
imat_max(iMatrix *M){
//prints out row and col index of first occurrence of max value
	int i, j, maxi=0, maxj=0;
	int maxx;
	maxx = -MAXINT;
	for(i=0; i<M->rows; i++){
		for(j=0; j<M->cols; j++){
			if(M->x[i][j] > maxx) {
				maxx = M->x[i][j];
				maxi = i; maxj = j;
			}
		}
	}
	print("%d %d\n", maxi, maxj);
	return 1;
}


iMatrix *
imat_transpose(iMatrix *M){
// matrix transposition

	iMatrix *new;
	int i, j;
	new = imat_alloc(M->cols, M->rows);
	for(i=0; i<M->rows; i++){
		for(j=0; j<M->cols; j++){
			new->x[j][i] = M->x[i][j];
		}
	}
	return new;
}

int
sq_imat_transpose(iMatrix *M)
{
	iMatrix *tmp;
	int i, j;
	
	tmp= imat_alloc(M->cols, M->rows);
	for(i=0; i<M->rows; i++){
		for(j=0; j<M->cols; j++){
			tmp->x[j][i] = M->x[i][j];
		}
	}
	for(i=0; i<M->rows; i++){
		for(j=0; j<M->cols; j++){
			M->x[i][j] = tmp->x[i][j];
		}
	}
	return 1;
}


void 
imat_ident(iMatrix *M){
// store identity matrix in M

	int i, j;
	for(i=0; i<M->rows; i++)
		for(j=0; j<M->cols; j++)
			if(i==j)
				M->x[j][i] = 1;
			else
				M->x[j][i] = 0;
}

void 
imat_add(iMatrix *a, iMatrix *b, iMatrix *out)
// matrix addition
{	int	i, j;
	for ( i=0; i<a->rows; i++ )
		for ( j=0; j<a->cols; j++ )
			out->x[i][j] = a->x[i][j]+b->x[i][j];
}


void 
imat_sub(iMatrix *a, iMatrix *b, iMatrix *out)
// matrix subtraction

{
	int	i, j;
	for ( i=0; i<a->rows; i++ )
		for ( j=0; j<a->cols; j++ )
			out->x[i][j] = a->x[i][j]-b->x[i][j];
}

void 
imat_mult(iMatrix *a, iMatrix *b, iMatrix *out)
// matrix multiplication
{
	int	i, j, k;
	double sum;
	for ( i=0; i<a->rows; i++ )
		for  ( j=0; j<b->cols; j++ )
		{
			sum = 0.0;
			for ( k=0; k<a->cols; k++ )
				sum += a->x[i][k]*b->x[k][j];
			out->x[i][j] = sum;
		}
}
